@@ -50,6 +50,11 @@
 #' s2_centroid("POLYGON ((0 0, 10 0, 10 10, 0 10, 0 0))")
 #' s2_centroid("LINESTRING (0 0, 10 0)")
 #'
+#' # s2_point_on_surface guarantees a point on surface
+#' # Note: this is not the same as st_point_on_surface
+#' s2_centroid("POLYGON ((0 0, 10 0, 1 1, 0 10, 0 0))")
+#' s2_point_on_surface("POLYGON ((0 0, 10 0, 1 1, 0 10, 0 0))")
+#'
 #' # returns the unweighted centroid of the entire input
 #' s2_centroid_agg(c("POINT (0 0)", "POINT (10 0)"))
 #'
@@ -94,6 +99,14 @@
 #'   s2_options(snap = s2_snap_level(30))
 #' )
 #'
+#' # s2_convex_hull_agg builds the convex hull of a list of geometries
+#' s2_convex_hull_agg(
+#'   c(
+#'     "POLYGON ((0 0, 10 0, 10 10, 0 10, 0 0))",
+#'     "POLYGON ((5 5, 15 5, 15 15, 5 15, 5 5))"
+#'   )
+#' )
+#'
 #' # use s2_union_agg() to aggregate geographies in a vector
 #' s2_coverage_union_agg(
 #'   c(
@@ -107,49 +120,50 @@
 #' # snap to grid rounds coordinates to a specified grid size
 #' s2_snap_to_grid("POINT (0.333333333333 0.666666666666)", 1e-2)
 #'
+#'
 s2_boundary <- function(x) {
-  new_s2_xptr(cpp_s2_boundary(as_s2_geography(x)), "s2_geography")
+  new_s2_geography(cpp_s2_boundary(as_s2_geography(x)))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_centroid <- function(x) {
-  new_s2_xptr(cpp_s2_centroid(as_s2_geography(x)), "s2_geography")
+  new_s2_geography(cpp_s2_centroid(as_s2_geography(x)))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_closest_point <- function(x, y) {
   recycled <- recycle_common(as_s2_geography(x), as_s2_geography(y))
-  new_s2_xptr(cpp_s2_closest_point(recycled[[1]], recycled[[2]]), "s2_geography")
+  new_s2_geography(cpp_s2_closest_point(recycled[[1]], recycled[[2]]))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_minimum_clearance_line_between <- function(x, y) {
   recycled <- recycle_common(as_s2_geography(x), as_s2_geography(y))
-  new_s2_xptr(cpp_s2_minimum_clearance_line_between(recycled[[1]], recycled[[2]]), "s2_geography")
+  new_s2_geography(cpp_s2_minimum_clearance_line_between(recycled[[1]], recycled[[2]]))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_difference <- function(x, y, options = s2_options()) {
   recycled <- recycle_common(as_s2_geography(x), as_s2_geography(y))
-  new_s2_xptr(cpp_s2_difference(recycled[[1]], recycled[[2]], options), "s2_geography")
+  new_s2_geography(cpp_s2_difference(recycled[[1]], recycled[[2]], options))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_sym_difference <- function(x, y, options = s2_options()) {
   recycled <- recycle_common(as_s2_geography(x), as_s2_geography(y))
-  new_s2_xptr(cpp_s2_sym_difference(recycled[[1]], recycled[[2]], options), "s2_geography")
+  new_s2_geography(cpp_s2_sym_difference(recycled[[1]], recycled[[2]], options))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_intersection <- function(x, y, options = s2_options()) {
   recycled <- recycle_common(as_s2_geography(x), as_s2_geography(y))
-  new_s2_xptr(cpp_s2_intersection(recycled[[1]], recycled[[2]], options), "s2_geography")
+  new_s2_geography(cpp_s2_intersection(recycled[[1]], recycled[[2]], options))
 }
 
 #' @rdname s2_boundary
@@ -158,10 +172,10 @@ s2_union <- function(x, y = NULL, options = s2_options()) {
   x <- as_s2_geography(x)
 
   if (is.null(y)) {
-    new_s2_xptr(cpp_s2_unary_union(x, options), "s2_geography")
+    new_s2_geography(cpp_s2_unary_union(x, options))
   } else {
     recycled <- recycle_common(x, as_s2_geography(y))
-    new_s2_xptr(cpp_s2_union(recycled[[1]], recycled[[2]], options), "s2_geography")
+    new_s2_geography(cpp_s2_union(recycled[[1]], recycled[[2]], options))
   }
 }
 
@@ -186,41 +200,52 @@ s2_simplify <- function(x, tolerance, radius = s2_earth_radius_meters()) {
 #' @rdname s2_boundary
 #' @export
 s2_rebuild <- function(x, options = s2_options()) {
-  new_s2_xptr(cpp_s2_rebuild(as_s2_geography(x), options), "s2_geography")
+  new_s2_geography(cpp_s2_rebuild(as_s2_geography(x), options))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_buffer_cells <- function(x, distance, max_cells = 1000, min_level = -1,
                             radius = s2_earth_radius_meters()) {
-  recycled <- recycle_common(as_s2_geography(x), distance / radius)
-  new_s2_xptr(cpp_s2_buffer_cells(recycled[[1]], recycled[[2]], max_cells, min_level), "s2_geography")
+  recycled <- recycle_common(as_s2_geography(x), distance / radius, max_cells, min_level)
+  new_s2_geography(cpp_s2_buffer_cells(recycled[[1]], recycled[[2]], recycled[[3]], recycled[[4]]))
+}
+
+#' @rdname s2_boundary
+#' @export
+s2_convex_hull <- function(x) {
+  new_s2_geography(cpp_s2_convex_hull(as_s2_geography(x)))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_centroid_agg <- function(x, na.rm = FALSE) {
-  new_s2_xptr(cpp_s2_centroid_agg(as_s2_geography(x), naRm = na.rm), "s2_geography")
+  new_s2_geography(cpp_s2_centroid_agg(as_s2_geography(x), naRm = na.rm))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_coverage_union_agg <- function(x, options = s2_options(), na.rm = FALSE) {
-  new_s2_xptr(cpp_s2_coverage_union_agg(as_s2_geography(x), options, na.rm), "s2_geography")
+  new_s2_geography(cpp_s2_coverage_union_agg(as_s2_geography(x), options, na.rm))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_rebuild_agg <- function(x, options = s2_options(), na.rm = FALSE) {
-  new_s2_xptr(cpp_s2_rebuild_agg(as_s2_geography(x), options, na.rm), "s2_geography")
+  new_s2_geography(cpp_s2_rebuild_agg(as_s2_geography(x), options, na.rm))
 }
 
 #' @rdname s2_boundary
 #' @export
 s2_union_agg <- function(x, options = s2_options(), na.rm = FALSE) {
-  new_s2_xptr(cpp_s2_union_agg(s2_union(x, options = options), options, na.rm), "s2_geography")
+  new_s2_geography(cpp_s2_union_agg(s2_union(x, options = options), options, na.rm))
 }
 
+#' @rdname s2_boundary
+#' @export
+s2_convex_hull_agg <- function(x, na.rm = FALSE) {
+  new_s2_geography(cpp_s2_convex_hull_agg(as_s2_geography(x), na.rm))
+}
 
 #' Linear referencing
 #'
@@ -252,9 +277,8 @@ s2_union_agg <- function(x, options = s2_options(), na.rm = FALSE) {
 s2_interpolate <- function(x, distance, radius = s2_earth_radius_meters()) {
   recycled <- recycle_common(as_s2_geography(x), distance / radius)
   length <- cpp_s2_length(recycled[[1]])
-  new_s2_xptr(
-    cpp_s2_interpolate_normalized(recycled[[1]], distance / radius / length),
-    "s2_geography"
+  new_s2_geography(
+    cpp_s2_interpolate_normalized(recycled[[1]], distance / radius / length)
   )
 }
 
@@ -262,8 +286,13 @@ s2_interpolate <- function(x, distance, radius = s2_earth_radius_meters()) {
 #' @export
 s2_interpolate_normalized <- function(x, distance_normalized) {
   recycled <- recycle_common(as_s2_geography(x), distance_normalized)
-  new_s2_xptr(
-    cpp_s2_interpolate_normalized(recycled[[1]], distance_normalized),
-    "s2_geography"
+  new_s2_geography(
+    cpp_s2_interpolate_normalized(recycled[[1]], distance_normalized)
   )
 }
+
+#' @rdname s2_boundary
+#' @export
+s2_point_on_surface <- function(x, na.rm = FALSE) {
+  new_s2_geography(cpp_s2_point_on_surface(as_s2_geography(x)))
+}
