@@ -9,9 +9,7 @@
 #include "s2/s2cell.h"
 #include "s2/s2latlng.h"
 
-#include "point-geography.h"
-#include "polyline-geography.h"
-#include "polygon-geography.h"
+#include "geography.h"
 
 #include <Rcpp.h>
 using namespace Rcpp;
@@ -101,8 +99,8 @@ public:
       return output;
     } else {
       std::stringstream err;
-      err << 
-        "Can't recycle vectors of size " << cellIdVector1.size() << 
+      err <<
+        "Can't recycle vectors of size " << cellIdVector1.size() <<
         " and " << cellIdVector2.size() <<
         " to a common length.";
       stop(err.str());
@@ -137,7 +135,7 @@ NumericVector cpp_s2_cell_from_string(CharacterVector cellString) {
       ptrCellId[i] = S2CellId::FromToken(as<std::string>(cellString[i])).id();
     }
   }
-  
+
   cellId.attr("class") = CharacterVector::create("s2_cell", "wk_vctr");
   return cellId;
 }
@@ -201,11 +199,35 @@ List cpp_s2_cell_to_lnglat(NumericVector cellId) {
     return List::create(_["x"] = lng, _["y"] = lat);
 }
 
+// [[Rcpp::export]]
+List cpp_s2_cell_to_cell_union(NumericVector cellId) {
+  R_xlen_t size = cellId.size();
+  CharacterVector cls = CharacterVector::create("s2_cell", "wk_vctr");
+  List out(size);
+
+  for (R_xlen_t i = 0; i < size; i++) {
+    if ((i % 1000) == 0) {
+      Rcpp::checkUserInterrupt();
+    }
+
+    if (R_IsNA(cellId[i])) {
+      out[i] = R_NilValue;
+    } else {
+      NumericVector item = NumericVector::create(cellId[i]);
+      item.attr("class") = cls;
+      out[i] = item;
+    }
+  }
+
+  out.attr("class") = CharacterVector::create("s2_cell_union", "wk_vctr");
+  return out;
+}
+
 // [[Rcpp::export]]
 LogicalVector cpp_s2_cell_is_na(NumericVector cellIdVector) {
   LogicalVector out(cellIdVector.size());
   for (R_xlen_t i = 0; i < cellIdVector.size(); i++) {
-    out = R_IsNA(cellIdVector[i]);
+    out[i] = R_IsNA(cellIdVector[i]);
   }
   return out;
 }
@@ -240,11 +262,11 @@ NumericVector cpp_s2_cell_range(NumericVector cellIdVector, bool naRm) {
       dataRange.second = data[i];
       break;
     }
-    
+
     if (!R_IsNA(cellIdVector[i]) && (data[i] < dataRange.first)) {
       dataRange.first = data[i];
     }
-    
+
     if (!R_IsNA(cellIdVector[i]) && (data[i] > dataRange.second)) {
       dataRange.second = data[i];
     }
@@ -332,7 +354,7 @@ List cpp_s2_cell_center(NumericVector cellIdVector) {
   class Op: public UnaryS2CellOperator<List, SEXP> {
     SEXP processCell(S2CellId cellId, R_xlen_t i) {
       if (cellId.is_valid()) {
-        return XPtr<PointGeography>(new PointGeography(cellId.ToPoint()));
+        return RGeography::MakeXPtr(RGeography::MakePoint(cellId.ToPoint()));
       } else {
         return R_NilValue;
       }
@@ -341,7 +363,7 @@ List cpp_s2_cell_center(NumericVector cellIdVector) {
 
   Op op;
   List result = op.processVector(cellIdVector);
-  result.attr("class") = CharacterVector::create("s2_geography", "s2_xptr");
+  result.attr("class") = CharacterVector::create("s2_geography", "wk_vctr");
   return result;
 }
 
@@ -350,7 +372,8 @@ List cpp_s2_cell_polygon(NumericVector cellIdVector) {
   class Op: public UnaryS2CellOperator<List, SEXP> {
     SEXP processCell(S2CellId cellId, R_xlen_t i) {
       if (cellId.is_valid()) {
-        return XPtr<PolygonGeography>(new PolygonGeography(absl::make_unique<S2Polygon>(S2Cell(cellId))));
+        auto poly = absl::make_unique<S2Polygon>(S2Cell(cellId));
+        return RGeography::MakeXPtr(RGeography::MakePolygon(std::move(poly)));
       } else {
         return R_NilValue;
       }
@@ -359,7 +382,7 @@ List cpp_s2_cell_polygon(NumericVector cellIdVector) {
 
   Op op;
   List result = op.processVector(cellIdVector);
-  result.attr("class") = CharacterVector::create("s2_geography", "s2_xptr");
+  result.attr("class") = CharacterVector::create("s2_geography", "wk_vctr");
   return result;
 }
 
@@ -368,7 +391,7 @@ List cpp_s2_cell_vertex(NumericVector cellIdVector, IntegerVector k) {
   class Op: public UnaryS2CellOperator<List, SEXP> {
     SEXP processCell(S2CellId cellId, R_xlen_t i) {
       if (cellId.is_valid() && (this->k[i] >= 0)) {
-        return XPtr<PointGeography>(new PointGeography(S2Cell(cellId).GetVertex(this->k[i])));
+        return RGeography::MakeXPtr(RGeography::MakePoint(S2Cell(cellId).GetVertex(this->k[i])));
       } else {
         return R_NilValue;
       }
@@ -381,7 +404,7 @@ List cpp_s2_cell_vertex(NumericVector cellIdVector, IntegerVector k) {
   Op op;
   op.k = k;
   List result = op.processVector(cellIdVector);
-  result.attr("class") = CharacterVector::create("s2_geography", "s2_xptr");
+  result.attr("class") = CharacterVector::create("s2_geography", "wk_vctr");
   return result;
 }
 
@@ -581,7 +604,7 @@ NumericVector cpp_s2_cell_cummin(NumericVector cellIdVector) {
 LogicalVector cpp_s2_cell_eq(NumericVector cellIdVector1, NumericVector cellIdVector2) {
   class Op: public BinaryS2CellOperator<LogicalVector, int> {
     int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
-      if (R_IsNA(reinterpret_double(cellId1.id())) || 
+      if (R_IsNA(reinterpret_double(cellId1.id())) ||
             R_IsNA(reinterpret_double(cellId2.id()))) {
         return NA_LOGICAL;
       } else {
@@ -598,7 +621,7 @@ LogicalVector cpp_s2_cell_eq(NumericVector cellIdVector1, NumericVector cellIdVe
 LogicalVector cpp_s2_cell_neq(NumericVector cellIdVector1, NumericVector cellIdVector2) {
   class Op: public BinaryS2CellOperator<LogicalVector, int> {
     int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
-      if (R_IsNA(reinterpret_double(cellId1.id())) || 
+      if (R_IsNA(reinterpret_double(cellId1.id())) ||
             R_IsNA(reinterpret_double(cellId2.id()))) {
         return NA_LOGICAL;
       } else {
@@ -615,7 +638,7 @@ LogicalVector cpp_s2_cell_neq(NumericVector cellIdVector1, NumericVector cellIdV
 LogicalVector cpp_s2_cell_lt(NumericVector cellIdVector1, NumericVector cellIdVector2) {
   class Op: public BinaryS2CellOperator<LogicalVector, int> {
     int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
-      if (R_IsNA(reinterpret_double(cellId1.id())) || 
+      if (R_IsNA(reinterpret_double(cellId1.id())) ||
             R_IsNA(reinterpret_double(cellId2.id()))) {
         return NA_LOGICAL;
       } else {
@@ -632,7 +655,7 @@ LogicalVector cpp_s2_cell_lt(NumericVector cellIdVector1, NumericVector cellIdVe
 LogicalVector cpp_s2_cell_lte(NumericVector cellIdVector1, NumericVector cellIdVector2) {
   class Op: public BinaryS2CellOperator<LogicalVector, int> {
     int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
-      if (R_IsNA(reinterpret_double(cellId1.id())) || 
+      if (R_IsNA(reinterpret_double(cellId1.id())) ||
             R_IsNA(reinterpret_double(cellId2.id()))) {
         return NA_LOGICAL;
       } else {
@@ -649,7 +672,7 @@ LogicalVector cpp_s2_cell_lte(NumericVector cellIdVector1, NumericVector cellIdV
 LogicalVector cpp_s2_cell_gte(NumericVector cellIdVector1, NumericVector cellIdVector2) {
   class Op: public BinaryS2CellOperator<LogicalVector, int> {
     int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
-      if (R_IsNA(reinterpret_double(cellId1.id())) || 
+      if (R_IsNA(reinterpret_double(cellId1.id())) ||
             R_IsNA(reinterpret_double(cellId2.id()))) {
         return NA_LOGICAL;
       } else {
@@ -666,7 +689,7 @@ LogicalVector cpp_s2_cell_gte(NumericVector cellIdVector1, NumericVector cellIdV
 LogicalVector cpp_s2_cell_gt(NumericVector cellIdVector1, NumericVector cellIdVector2) {
   class Op: public BinaryS2CellOperator<LogicalVector, int> {
     int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
-      if (R_IsNA(reinterpret_double(cellId1.id())) || 
+      if (R_IsNA(reinterpret_double(cellId1.id())) ||
             R_IsNA(reinterpret_double(cellId2.id()))) {
         return NA_LOGICAL;
       } else {
@@ -742,3 +765,48 @@ NumericVector cpp_s2_cell_max_distance(NumericVector cellIdVector1, NumericVecto
   Op op;
   return op.processVector(cellIdVector1, cellIdVector2);
 }
+
+// [[Rcpp::export]]
+IntegerVector cpp_s2_cell_common_ancestor_level(NumericVector cellIdVector1, NumericVector cellIdVector2) {
+  class Op: public BinaryS2CellOperator<IntegerVector, int> {
+    int processCell(S2CellId cellId1, S2CellId cellId2, R_xlen_t i) {
+      if (cellId1.is_valid() && cellId2.is_valid()) {
+        return cellId1.GetCommonAncestorLevel(cellId2);
+      } else {
+        return NA_INTEGER;
+      }
+    }
+  };
+
+  Op op;
+  return op.processVector(cellIdVector1, cellIdVector2);
+}
+
+// [[Rcpp::export]]
+int cpp_s2_cell_common_ancestor_level_agg(NumericVector cellId) {
+  R_xlen_t size = cellId.size();
+
+  if (size == 0) {
+    return NA_INTEGER;
+  }
+
+  double* ptrDouble = REAL(cellId);
+  uint64_t* ptrCellId = (uint64_t*) ptrDouble;
+
+  S2CellId cellIdCommon(ptrCellId[0]);
+
+  for (R_xlen_t i = 1; i < size; i++) {
+    if ((i % 1000) == 0) {
+      Rcpp::checkUserInterrupt(); // # nocov
+    }
+
+    int commonLevel = cellIdCommon.GetCommonAncestorLevel(S2CellId(ptrCellId[i]));
+    if (commonLevel == -1) {
+      return -1;
+    }
+
+    cellIdCommon = cellIdCommon.parent(commonLevel);
+  }
+
+  return cellIdCommon.level();
+}
