@@ -3,7 +3,6 @@
 #include "s2/s2closest_edge_query.h"
 #include "s2/s2latlng_rect.h"
 #include "s2/s2polygon.h"
-#include "s2/s2testing.h"
 #include "s2/s2builderutil_snap_functions.h"
 #include "s2/s2shape_index_buffered_region.h"
 
@@ -156,7 +155,7 @@ LogicalVector cpp_s2_prepared_dwithin(List geog1, List geog2, NumericVector dist
       iterator.Init(&feature1->Index().ShapeIndex());
       bool may_intersect_buffer = false;
       for (const S2CellId& query_cell: covering) {
-          if (iterator.Locate(query_cell) != S2ShapeIndex::CellRelation::DISJOINT) {
+          if (iterator.Locate(query_cell) != S2CellRelation::DISJOINT) {
             may_intersect_buffer = true;
             break;
           }
