@@ -17,10 +17,26 @@
 
 #include "s2/encoded_s2point_vector.h"
 
+#include <cstddef>
+
+#include <algorithm>
+#include <limits>
+#include <vector>
+
+#include "s2/base/integral_types.h"
+#include "s2/base/port.h"
 #include "absl/base/internal/unaligned_access.h"
+#include "absl/numeric/bits.h"
+#include "absl/types/span.h"
 #include "s2/util/bits/bits.h"
+#include "s2/util/coding/coder.h"
+#include "s2/util/coding/varint.h"
+#include "s2/encoded_string_vector.h"
+#include "s2/encoded_uint_vector.h"
 #include "s2/s2cell_id.h"
+#include "s2/s2coder.h"
 #include "s2/s2coords.h"
+#include "s2/s2point.h"
 
 using absl::MakeSpan;
 using absl::Span;
@@ -33,7 +49,8 @@ namespace s2coding {
 // Like util_bits::InterleaveUint32, but interleaves bit pairs rather than
 // individual bits.  This format is faster to decode than the fully interleaved
 // format, and produces the same results for our use case.
-inline uint64 InterleaveUint32BitPairs(const uint32 val0, const uint32 val1) {
+inline uint64 InterleaveUint32BitPairs(const uint32 val0,
+                                         const uint32 val1) {
   uint64 v0 = val0, v1 = val1;
   v0 = (v0 | (v0 << 16)) & 0x0000ffff0000ffff;
   v1 = (v1 | (v1 << 16)) & 0x0000ffff0000ffff;
@@ -50,8 +67,8 @@ inline uint64 InterleaveUint32BitPairs(const uint32 val0, const uint32 val1) {
 // uses a lookup table.  The speed advantage is expected to be even larger in
 // code that mixes bit interleaving with other significant operations since it
 // doesn't require keeping a 256-byte lookup table in the L1 data cache.
-inline void DeinterleaveUint32BitPairs(uint64 code,
-                                       uint32 *val0, uint32 *val1) {
+inline void DeinterleaveUint32BitPairs(uint64 code, uint32* val0,
+                                       uint32* val1) {
   uint64 v0 = code, v1 = code >> 2;
   v0 &= 0x3333333333333333;
   v0 |= v0 >> 2;
@@ -96,7 +113,7 @@ void EncodeS2PointVector(Span<const S2Point> points, CodingHint hint,
       return EncodeS2PointVectorCompact(points, encoder);
 
     default:
-      S2_LOG(DFATAL) << "Unknown CodingHint: " << static_cast<int>(hint);
+      S2_LOG(ERROR) << "Unknown CodingHint: " << static_cast<int>(hint);
   }
 }
 
@@ -105,7 +122,7 @@ bool EncodedS2PointVector::Init(Decoder* decoder) {
 
   // Peek at the format but don't advance the decoder; the format-specific
   // Init functions will do that.
-  format_ = static_cast<Format>(*decoder->ptr() & kEncodingFormatMask);
+  format_ = static_cast<Format>(*decoder->skip(0) & kEncodingFormatMask);
   switch (format_) {
     case UNCOMPRESSED:
       return InitUncompressedFormat(decoder);
@@ -121,12 +138,29 @@ bool EncodedS2PointVector::Init(Decoder* decoder) {
 vector<S2Point> EncodedS2PointVector::Decode() const {
   vector<S2Point> points;
   points.reserve(size_);
-  for (int i = 0; i < size_; ++i) {
+  for (size_t i = 0; i < size_; ++i) {
     points.push_back((*this)[i]);
   }
   return points;
 }
 
+// The encoding must be identical to EncodeS2PointVector().
+void EncodedS2PointVector::Encode(Encoder* encoder) const {
+  switch (format_) {
+    case UNCOMPRESSED:
+      EncodeS2PointVectorFast(MakeSpan(uncompressed_.points, size_), encoder);
+      break;
+
+    case CELL_IDS: {
+      // This is a full decode/encode dance, and not at all efficient.
+      EncodeS2PointVectorCompact(Decode(), encoder);
+      break;
+    }
+
+    default:
+      S2_LOG(FATAL) << "Unknown Format: " << static_cast<int>(format_);
+  }
+}
 
 //////////////////////////////////////////////////////////////////////////////
 //                     UNCOMPRESSED Encoding Format
@@ -147,7 +181,7 @@ void EncodeS2PointVectorFast(Span<const S2Point> points, Encoder* encoder) {
   // This is followed by an array of S2Points in little-endian order.
   encoder->Ensure(Varint::kMax64 + points.size() * sizeof(S2Point));
   uint64 size_format = (points.size() << kEncodingFormatBits |
-                        EncodedS2PointVector::UNCOMPRESSED);
+                          EncodedS2PointVector::UNCOMPRESSED);
   encoder->put_varint64(size_format);
   encoder->putn(points.data(), points.size() * sizeof(S2Point));
 }
@@ -155,15 +189,15 @@ void EncodeS2PointVectorFast(Span<const S2Point> points, Encoder* encoder) {
 bool EncodedS2PointVector::InitUncompressedFormat(Decoder* decoder) {
 #if !defined(IS_LITTLE_ENDIAN) || defined(__arm__) || \
   defined(ABSL_INTERNAL_NEED_ALIGNED_LOADS)
-  // TODO(ericv): Make this work on platforms that don't support unaligned
-  // 64-bit little-endian reads, e.g. by falling back to
+  // TODO(b/231674214): Make this work on platforms that don't support
+  // unaligned 64-bit little-endian reads, e.g. by falling back to
   //
   //   bit_cast<double>(little_endian::Load64()).
   //
   // Maybe the compiler is smart enough that we can do this all the time,
   // but more likely we will need two cases using the #ifdef above.
   // (Note that even ARMv7 does not support unaligned 64-bit loads.)
-  S2_LOG(DFATAL) << "Needs architecture with 64-bit little-endian unaligned loads";
+  S2_LOG(ERROR) << "Needs architecture with 64-bit little-endian unaligned loads";
   return false;
 #endif
 
@@ -179,7 +213,7 @@ bool EncodedS2PointVector::InitUncompressedFormat(Decoder* decoder) {
   size_t bytes = size_t{size_} * sizeof(S2Point);
   if (decoder->avail() < bytes) return false;
 
-  uncompressed_.points = reinterpret_cast<const S2Point*>(decoder->ptr());
+  uncompressed_.points = reinterpret_cast<const S2Point*>(decoder->skip(0));
   decoder->skip(bytes);
   return true;
 }
@@ -194,7 +228,7 @@ bool EncodedS2PointVector::InitUncompressedFormat(Decoder* decoder) {
 struct CellPoint {
   // Constructor necessary in order to narrow "int" arguments to "int8".
   CellPoint(int level, int face, uint32 si, uint32 ti)
-    : level(level), face(face), si(si), ti(ti) {}
+      : level(level), face(face), si(si), ti(ti) {}
 
   int8 level, face;
   uint32 si, ti;
@@ -205,12 +239,12 @@ struct CellPoint {
 // Block sizes of 4, 8, 16, and 32 were tested and kBlockSize == 16 seems to
 // offer the best compression.  (Note that kBlockSize == 32 requires some code
 // modifications which have since been removed.)
-constexpr int kBlockShift = 4;
-constexpr size_t kBlockSize = 1 << kBlockShift;
+static constexpr int kBlockShift = 4;
+static constexpr size_t kBlockSize = 1 << kBlockShift;
 
 // Used to indicate that a point must be encoded as an exception (a 24-byte
 // S2Point) rather than as an S2CellId.
-constexpr uint64 kException = ~0ULL;
+static constexpr uint64 kException = ~0ULL;
 
 // Represents the encoding parameters to be used for a given block (consisting
 // of kBlockSize encodable 64-bit values).  See below.
@@ -221,9 +255,7 @@ struct BlockCode {
 };
 
 // Returns a bit mask with "n" low-order 1 bits, for 0 <= n <= 64.
-inline uint64 BitMask(int n) {
-  return (n == 0) ? 0 : (~0ULL >> (64 - n));
-}
+inline uint64 BitMask(int n) { return (n == 0) ? 0 : (~0ULL >> (64 - n)); }
 
 // Returns the maximum number of bits per value at the given S2CellId level.
 inline int MaxBitsForLevel(int level) {
@@ -240,9 +272,9 @@ inline int BaseShift(int level, int base_bits) {
 // Forward declarations.
 int ChooseBestLevel(Span<const S2Point> points, vector<CellPoint>* cell_points);
 vector<uint64> ConvertCellsToValues(const vector<CellPoint>& cell_points,
-                                    int level, bool* have_exceptions);
-uint64 ChooseBase(const vector<uint64>& values, int level, bool have_exceptions,
-                  int* base_bits);
+                                      int level, bool* have_exceptions);
+uint64 ChooseBase(const vector<uint64>& values, int level,
+                    bool have_exceptions, int* base_bits);
 BlockCode GetBlockCode(Span<const uint64> values, uint64 base,
                        bool have_exceptions);
 
@@ -292,9 +324,9 @@ void EncodeS2PointVectorCompact(Span<const S2Point> points, Encoder* encoder) {
   // except that it is faster to decode and the spatial locality is not quite
   // as good.
   //
-  // The 64-bit values are divided into blocks of size 8, and then each value is
-  // encoded as the sum of a base value, a per-block offset, and a per-value
-  // delta within that block:
+  // The 64-bit values are divided into blocks of size kBlockSize, and then
+  // each value is encoded as the sum of a base value, a per-block offset, and
+  // a per-value delta within that block:
   //
   //   v[i,j] = base + offset[i] + delta[i, j]
   //
@@ -380,10 +412,11 @@ void EncodeS2PointVectorCompact(Span<const S2Point> points, Encoder* encoder) {
   //
   // If there are any points that could not be represented as S2CellIds, then
   // "have_exceptions" in the header is true.  In that case the delta values
-  // within each block are encoded as (delta + 8), and values 0-7 are used to
-  // represent exceptions.  If a block has exceptions, they are encoded
-  // immediately following the array of deltas, and are referenced by encoding
-  // the corresponding exception index (0-7) as the delta.
+  // within each block are encoded as (delta + kBlockSize), and values
+  // 0...kBlockSize-1 are used to represent exceptions.  If a block has
+  // exceptions, they are encoded immediately following the array of deltas,
+  // and are referenced by encoding the corresponding exception index
+  // 0...kBlockSize-1 as the delta.
   //
   // TODO(ericv): A vector containing a single leaf cell is currently encoded as
   // 13 bytes (2 byte header, 7 byte base, 1 byte block count, 1 byte block
@@ -394,7 +427,7 @@ void EncodeS2PointVectorCompact(Span<const S2Point> points, Encoder* encoder) {
   // (3 bits), followed by the S2CellId bytes.  The extra 2 header bits could be
   // used to store single points using other encodings, e.g. E7.
   //
-  // If we wind up using 8-value blocks, we could also use the extra bit in the
+  // If we had used 8-value blocks, we could have used the extra bit in the
   // first byte of the header to indicate that there is only one value, and
   // then skip the 2nd byte of header and the EncodedStringVector.  But this
   // would be messy because it also requires special cases while decoding.
@@ -414,8 +447,8 @@ void EncodeS2PointVectorCompact(Span<const S2Point> points, Encoder* encoder) {
   //
   // TODO(ericv): Benchmark using shifted S2CellIds instead.
   bool have_exceptions;
-  vector<uint64> values = ConvertCellsToValues(cell_points, level,
-                                               &have_exceptions);
+  vector<uint64> values =
+      ConvertCellsToValues(cell_points, level, &have_exceptions);
 
   // 3. Choose the global encoding parameter "base" (consisting of the bit
   // prefix shared by all values to be encoded).
@@ -443,10 +476,7 @@ void EncodeS2PointVectorCompact(Span<const S2Point> points, Encoder* encoder) {
   // Now we encode the contents of each block.
   StringVectorEncoder blocks;
   vector<S2Point> exceptions;
-  uint64 offset_bytes_sum = 0;
-  uint64 delta_nibbles_sum = 0;
-  uint64 exceptions_sum = 0;
-  for (int i = 0; i < values.size(); i += kBlockSize) {
+  for (size_t i = 0; i < values.size(); i += kBlockSize) {
     int block_size = min(kBlockSize, values.size() - i);
     BlockCode code = GetBlockCode(MakeSpan(&values[i], block_size),
                                   base, have_exceptions);
@@ -515,9 +545,6 @@ void EncodeS2PointVectorCompact(Span<const S2Point> points, Encoder* encoder) {
       block->Ensure(exceptions_bytes);
       block->putn(exceptions.data(), exceptions_bytes);
     }
-    offset_bytes_sum += offset_bytes;
-    delta_nibbles_sum += delta_nibbles;
-    exceptions_sum += num_exceptions;
   }
   blocks.Encode(encoder);
 }
@@ -569,7 +596,7 @@ int ChooseBestLevel(Span<const S2Point> points,
 // indicated by the value "kException".  "have_exceptions" is set to indicate
 // whether any exceptions were present.
 vector<uint64> ConvertCellsToValues(const vector<CellPoint>& cell_points,
-                                    int level, bool* have_exceptions) {
+                                      int level, bool* have_exceptions) {
   vector<uint64> values;
   values.reserve(cell_points.size());
   *have_exceptions = false;
@@ -595,8 +622,8 @@ vector<uint64> ConvertCellsToValues(const vector<CellPoint>& cell_points,
   return values;
 }
 
-uint64 ChooseBase(const vector<uint64>& values, int level, bool have_exceptions,
-                  int* base_bits) {
+uint64 ChooseBase(const vector<uint64>& values, int level,
+                    bool have_exceptions, int* base_bits) {
   // Find the minimum and maximum non-exception values to be represented.
   uint64 v_min = kException, v_max = 0;
   for (auto v : values) {
@@ -619,8 +646,8 @@ uint64 ChooseBase(const vector<uint64>& values, int level, bool have_exceptions,
   // 2. The format only allows us to represent up to 7 bytes (56 bits) of
   // "base", so we need to ensure that "base" conforms to this requirement.
   int min_delta_bits = (have_exceptions || values.size() == 1) ? 8 : 4;
-  int excluded_bits = max(Bits::Log2Floor64(v_min ^ v_max) + 1,
-                          max(min_delta_bits, BaseShift(level, 56)));
+  int excluded_bits = max<int>(absl::bit_width(v_min ^ v_max),
+                               max(min_delta_bits, BaseShift(level, 56)));
   uint64 base = v_min & ~BitMask(excluded_bits);
 
   // Determine how many bytes are needed to represent this prefix.
@@ -645,8 +672,8 @@ uint64 ChooseBase(const vector<uint64>& values, int level, bool have_exceptions,
 
 // Returns true if the range of values [d_min, d_max] can be encoded using the
 // specified parameters (delta_bits, overlap_bits, and have_exceptions).
-bool CanEncode(uint64 d_min, uint64 d_max, int delta_bits,
-               int overlap_bits, bool have_exceptions) {
+bool CanEncode(uint64 d_min, uint64 d_max, int delta_bits, int overlap_bits,
+               bool have_exceptions) {
   // "offset" can't represent the lowest (delta_bits - overlap_bits) of d_min.
   d_min &= ~BitMask(delta_bits - overlap_bits);
 
@@ -721,7 +748,8 @@ BlockCode GetBlockCode(Span<const uint64> values, uint64 base,
   //
   // It is possible to show that this last example is the worst case, i.e.  we
   // do not need to consider increasing delta_bits or overlap_bits further.
-  int delta_bits = (max(1, Bits::Log2Floor64(b_max - b_min)) + 3) & ~3;
+  int delta_bits =
+      (max(1, static_cast<int>(absl::bit_width(b_max - b_min)) - 1) + 3) & ~3;
   int overlap_bits = 0;
   if (!CanEncode(b_min, b_max, delta_bits, 0, have_exceptions)) {
     if (CanEncode(b_min, b_max, delta_bits, 4, have_exceptions)) {
@@ -736,9 +764,12 @@ BlockCode GetBlockCode(Span<const uint64> values, uint64 base,
     }
   }
 
-  // Avoid wasting 4 bits of delta when the block size is 1.  This reduces the
-  // encoding size for single leaf cells by one byte.
-  if (values.size() == 1) {
+  // When the block size is 1 and no exceptions exist, we have delta_bits == 4
+  // and overlap_bits == 0 which wastes 4 bits.  We fix this below, which
+  // among other things reduces the encoding size for single leaf cells by one
+  // byte.  (Note that when exceptions exist, delta_bits == 8 and overlap_bits
+  // may be 0 or 4.  These cases are covered by the unit tests.)
+  if (values.size() == 1 && !have_exceptions) {
     S2_DCHECK(delta_bits == 4 && overlap_bits == 0);
     delta_bits = 8;
   }
@@ -798,7 +829,8 @@ S2Point EncodedS2PointVector::DecodeCellIdsFormat(int i) const {
 
   // Decode the offset for this block.
   int offset_shift = (delta_nibbles - overlap_nibbles) << 2;
-  uint64 offset = GetUintWithLength<uint64>(ptr, offset_bytes) << offset_shift;
+  uint64 offset = GetUintWithLength<uint64>(ptr, offset_bytes)
+                    << offset_shift;
   ptr += offset_bytes;
 
   // Decode the delta for the requested value.
