@@ -98,6 +98,7 @@
 #define S2_S2COORDS_H_
 
 #include <algorithm>
+#include <cfloat>
 #include <cmath>
 
 #include "s2/base/integral_types.h"
@@ -113,6 +114,13 @@
 // dimension of the surface, not the space it is embedded in).
 namespace S2 {
 
+// The maximum absolute error in U/V coordinates when converting from XYZ.
+//
+// The XYZ -> UV conversion is a single division per coordinate, which is
+// promised to be at most 0.5*DBL_EPSILON absolute error for values with
+// magnitude less than two.
+constexpr double kMaxXYZtoUVError = 0.5 * DBL_EPSILON;
+
 // This is the number of levels needed to specify a leaf cell.  This
 // constant is defined here so that the S2::Metric class and the conversion
 // functions below can be implemented without including s2cell_id.h.  Please
@@ -128,7 +136,7 @@ const int kLimitIJ = 1 << kMaxCellLevel;  // == S2CellId::kMaxSize
 unsigned const int kMaxSiTi = 1U << (kMaxCellLevel + 1);
 
 // Convert an s- or t-value to the corresponding u- or v-value.  This is
-// a non-linear transformation from [-1,1] to [-1,1] that attempts to
+// a non-linear transformation from [0,1] to [-1,1] that attempts to
 // make the cell sizes more uniform.
 double STtoUV(double s);
 
