@@ -17,55 +17,69 @@
 
 #include "s2/s2loop.h"
 
+#include <cstddef>
+
 #include <algorithm>
 #include <atomic>
 #include <bitset>
-#include <cfloat>
 #include <cmath>
-#include <set>
+#include <memory>
 #include <utility>
 #include <vector>
 
+#include "absl/container/flat_hash_set.h"
+#include "absl/flags/flag.h"
+#include "absl/types/span.h"
+#include "absl/utility/utility.h"
+
 #include "s2/base/commandlineflags.h"
 #include "s2/base/integral_types.h"
 #include "s2/base/logging.h"
 #include "s2/mutable_s2shape_index.h"
 #include "s2/r1interval.h"
+#include "s2/r2.h"
+#include "s2/r2rect.h"
 #include "s2/s1angle.h"
+#include "s2/s1chord_angle.h"
 #include "s2/s1interval.h"
 #include "s2/s2cap.h"
 #include "s2/s2cell.h"
-#include "s2/s2centroids.h"
+#include "s2/s2cell_id.h"
+#include "s2/s2cell_union.h"
 #include "s2/s2closest_edge_query.h"
 #include "s2/s2coords.h"
 #include "s2/s2crossing_edge_query.h"
 #include "s2/s2debug.h"
 #include "s2/s2edge_clipping.h"
 #include "s2/s2edge_crosser.h"
+#include "s2/s2edge_crossings.h"
 #include "s2/s2edge_distances.h"
 #include "s2/s2error.h"
+#include "s2/s2latlng_rect.h"
 #include "s2/s2latlng_rect_bounder.h"
-#include "s2/s2measures.h"
+#include "s2/s2loop_measures.h"
 #include "s2/s2padded_cell.h"
+#include "s2/s2point.h"
 #include "s2/s2point_compression.h"
 #include "s2/s2pointutil.h"
 #include "s2/s2predicates.h"
+#include "s2/s2region.h"
+#include "s2/s2shape.h"
 #include "s2/s2shape_index.h"
 #include "s2/s2shapeutil_visit_crossing_edge_pairs.h"
 #include "s2/s2wedge_relations.h"
-#include "absl/memory/memory.h"
-#include "absl/types/span.h"
-#include "s2/util/coding/coder.h"
 #include "s2/util/coding/coder.h"
 #include "s2/util/math/matrix3x3.h"
 
-using absl::make_unique;
+using absl::flat_hash_set;
 using absl::MakeSpan;
+using absl::Span;
+using std::make_unique;
 using std::pair;
-using std::set;
+using std::unique_ptr;
 using std::vector;
 
-DEFINE_bool(
+S2_DEFINE_bool(
     s2loop_lazy_indexing, true,
     "Build the S2ShapeIndex only when it is first needed.  This can save "
     "significant amounts of memory and time when geometry is constructed but "
@@ -74,7 +88,7 @@ DEFINE_bool(
 
 // The maximum number of vertices we'll allow when decoding a loop.
 // The default value of 50 million is about 30x bigger than the number of
-DEFINE_int32(
+S2_DEFINE_int32(
     s2polygon_decode_max_num_vertices, 50000000,
     "The upper limit on the number of loops that are allowed by the "
     "S2Polygon::Decode method.");
@@ -93,11 +107,54 @@ S2Loop::S2Loop() {
   // The loop is not valid until Init() is called.
 }
 
-S2Loop::S2Loop(const vector<S2Point>& vertices)
+#ifndef SWIG
+S2Loop::S2Loop(S2Loop&& b)
+    : S2Region(std::move(b)),
+      depth_(absl::exchange(b.depth_, 0)),
+      num_vertices_(absl::exchange(b.num_vertices_, 0)),
+      vertices_(std::move(b.vertices_)),
+      s2debug_override_(std::move(b.s2debug_override_)),
+      origin_inside_(std::move(b.origin_inside_)),
+      unindexed_contains_calls_(
+          b.unindexed_contains_calls_.exchange(0, std::memory_order_relaxed)),
+      bound_(std::move(b.bound_)),
+      subregion_bound_(std::move(b.subregion_bound_)),
+      index_(std::move(b.index_)) {
+  // Our index points to S2Loop::Shape instances which point back to S2Loop,
+  // we need to update those S2Loop pointers now that we've moved.
+  for (S2Shape* shape : index_) {
+    down_cast<Shape*>(shape)->loop_ = this;
+  }
+}
+
+S2Loop& S2Loop::operator=(S2Loop&& b) {
+  S2Region::operator=(static_cast<S2Region&&>(b));
+  depth_ = absl::exchange(b.depth_, 0);
+  num_vertices_ = absl::exchange(b.num_vertices_, 0);
+  vertices_ = std::move(b.vertices_);
+  s2debug_override_ = std::move(b.s2debug_override_);
+  origin_inside_ = std::move(b.origin_inside_);
+  unindexed_contains_calls_.store(
+      b.unindexed_contains_calls_.exchange(0, std::memory_order_relaxed),
+      std::memory_order_relaxed);
+  bound_ = std::move(b.bound_);
+  subregion_bound_ = std::move(b.subregion_bound_);
+  index_ = std::move(b.index_);
+
+  // Our index points to S2Loop::Shape instances which point back to S2Loop,
+  // we need to update those S2Loop pointers now that we've moved.
+  for (S2Shape* shape : index_) {
+    down_cast<Shape*>(shape)->loop_ = this;
+  }
+
+  return *this;
+}
+#endif
+
+S2Loop::S2Loop(Span<const S2Point> vertices)
   : S2Loop(vertices, S2Debug::ALLOW) {}
 
-S2Loop::S2Loop(const vector<S2Point>& vertices,
-               S2Debug override)
+S2Loop::S2Loop(Span<const S2Point> vertices, S2Debug override)
   : s2debug_override_(override) {
   Init(vertices);
 }
@@ -115,20 +172,18 @@ void S2Loop::ClearIndex() {
   index_.Clear();
 }
 
-void S2Loop::Init(const vector<S2Point>& vertices) {
+void S2Loop::Init(Span<const S2Point> vertices) {
   ClearIndex();
-  if (owns_vertices_) delete[] vertices_;
   num_vertices_ = vertices.size();
-  vertices_ = new S2Point[num_vertices_];
+  vertices_ = make_unique<S2Point[]>(num_vertices_);
   std::copy(vertices.begin(), vertices.end(), &vertices_[0]);
-  owns_vertices_ = true;
   InitOriginAndBound();
 }
 
 bool S2Loop::IsValid() const {
   S2Error error;
   if (FindValidationError(&error)) {
-    S2_LOG_IF(ERROR, FLAGS_s2debug) << error;
+    S2_LOG_IF(ERROR, absl::GetFlag(FLAGS_s2debug)) << error;
     return false;
   }
   return true;
@@ -197,28 +252,22 @@ void S2Loop::InitOriginAndBound() {
     // otherwise it is empty.
     origin_inside_ = (vertex(0).z() < 0);
   } else {
-    // Point containment testing is done by counting edge crossings starting
-    // at a fixed point on the sphere (S2::Origin()).  Historically this was
-    // important, but it is now no longer necessary, and it may be worthwhile
-    // experimenting with using a loop vertex as the reference point.  In any
-    // case, we need to know whether the reference point (S2::Origin) is
-    // inside or outside the loop before we can construct the S2ShapeIndex.
-    // We do this by first guessing that it is outside, and then seeing
-    // whether we get the correct containment result for vertex 1.  If the
-    // result is incorrect, the origin must be inside the loop.
+    // The brute force point containment algorithm works by counting edge
+    // crossings starting at a fixed reference point (chosen as S2::Origin()
+    // for historical reasons).  Loop initialization would be more efficient
+    // if we used a loop vertex such as vertex(0) as the reference point
+    // instead, however making this change would be a lot of work because
+    // origin_inside_ is currently part of the Encode() format.
     //
-    // A loop with consecutive vertices A,B,C contains vertex B if and only if
-    // the fixed vector R = S2::Ortho(B) is contained by the wedge ABC.  The
-    // wedge is closed at A and open at C, i.e. the point B is inside the loop
-    // if A=R but not if C=R.  This convention is required for compatibility
-    // with S2::VertexCrossing.  (Note that we can't use S2::Origin()
-    // as the fixed vector because of the possibility that B == S2::Origin().)
-    //
-    // TODO(ericv): Investigate using vertex(0) as the reference point.
-
+    // In any case, we initialize origin_inside_ by first guessing that it is
+    // outside, and then seeing whether we get the correct containment result
+    // for vertex 1.  If the result is incorrect, the origin must be inside
+    // the loop instead.  Note that the S2Loop is not necessarily valid and so
+    // we need to check the requirements of S2::AngleContainsVertex() first.
+    bool v1_inside = vertex(0) != vertex(1) && vertex(2) != vertex(1) &&
+                     S2::AngleContainsVertex(vertex(0), vertex(1), vertex(2));
     origin_inside_ = false;  // Initialize before calling Contains().
-    bool v1_inside = s2pred::OrderedCCW(S2::Ortho(vertex(1)), vertex(0),
-                                        vertex(2), vertex(1));
+
     // Note that Contains(S2Point) only does a bounds check once InitIndex()
     // has been called, so it doesn't matter that bound_ is undefined here.
     if (v1_inside != Contains(vertex(1))) {
@@ -276,10 +325,10 @@ void S2Loop::InitBound() {
 
 void S2Loop::InitIndex() {
   index_.Add(make_unique<Shape>(this));
-  if (!FLAGS_s2loop_lazy_indexing) {
+  if (!absl::GetFlag(FLAGS_s2loop_lazy_indexing)) {
     index_.ForceBuild();
   }
-  if (FLAGS_s2debug && s2debug_override_ == S2Debug::ALLOW) {
+  if (absl::GetFlag(FLAGS_s2debug) && s2debug_override_ == S2Debug::ALLOW) {
     // Note that FLAGS_s2debug is false in optimized builds (by default).
     S2_CHECK(IsValid());
   }
@@ -289,7 +338,6 @@ S2Loop::S2Loop(const S2Cell& cell)
     : depth_(0),
       num_vertices_(4),
       vertices_(new S2Point[num_vertices_]),
-      owns_vertices_(true),
       s2debug_override_(S2Debug::ALLOW),
       unindexed_contains_calls_(0) {
   for (int i = 0; i < 4; ++i) {
@@ -300,15 +348,12 @@ S2Loop::S2Loop(const S2Cell& cell)
   InitOriginAndBound();
 }
 
-S2Loop::~S2Loop() {
-  if (owns_vertices_) delete[] vertices_;
-}
+S2Loop::~S2Loop() = default;
 
 S2Loop::S2Loop(const S2Loop& src)
     : depth_(src.depth_),
       num_vertices_(src.num_vertices_),
-      vertices_(new S2Point[num_vertices_]),
-      owns_vertices_(true),
+      vertices_(make_unique<S2Point[]>(num_vertices_)),
       s2debug_override_(src.s2debug_override_),
       origin_inside_(src.origin_inside_),
       unindexed_contains_calls_(0),
@@ -352,18 +397,16 @@ bool S2Loop::IsNormalized() const {
 }
 
 void S2Loop::Normalize() {
-  S2_CHECK(owns_vertices_);
   if (!IsNormalized()) Invert();
   S2_DCHECK(IsNormalized());
 }
 
 void S2Loop::Invert() {
-  S2_CHECK(owns_vertices_);
   ClearIndex();
   if (is_empty_or_full()) {
     vertices_[0] = is_full() ? kEmptyVertex() : kFullVertex();
   } else {
-    std::reverse(vertices_, vertices_ + num_vertices());
+    std::reverse(&vertices_[0], &vertices_[num_vertices()]);
   }
   // origin_inside_ must be set correctly before building the S2ShapeIndex.
   origin_inside_ ^= true;
@@ -440,7 +483,7 @@ S2Cap S2Loop::GetCapBound() const {
 
 bool S2Loop::Contains(const S2Cell& target) const {
   MutableS2ShapeIndex::Iterator it(&index_);
-  S2ShapeIndex::CellRelation relation = it.Locate(target.id());
+  S2CellRelation relation = it.Locate(target.id());
 
   // If "target" is disjoint from all index cells, it is not contained.
   // Similarly, if "target" is subdivided into one or more index cells then it
@@ -448,7 +491,7 @@ bool S2Loop::Contains(const S2Cell& target) const {
   // intersect a sufficient number of edges.  (But note that if "target" itself
   // is an index cell then it may be contained, since it could be a cell with
   // no edges in the loop interior.)
-  if (relation != S2ShapeIndex::INDEXED) return false;
+  if (relation != S2CellRelation::INDEXED) return false;
 
   // Otherwise check if any edges intersect "target".
   if (BoundaryApproxIntersects(it, target)) return false;
@@ -459,14 +502,14 @@ bool S2Loop::Contains(const S2Cell& target) const {
 
 bool S2Loop::MayIntersect(const S2Cell& target) const {
   MutableS2ShapeIndex::Iterator it(&index_);
-  S2ShapeIndex::CellRelation relation = it.Locate(target.id());
+  S2CellRelation relation = it.Locate(target.id());
 
   // If "target" does not overlap any index cell, there is no intersection.
-  if (relation == S2ShapeIndex::DISJOINT) return false;
+  if (relation == S2CellRelation::DISJOINT) return false;
 
   // If "target" is subdivided into one or more index cells, there is an
   // intersection to within the S2ShapeIndex error bound (see Contains).
-  if (relation == S2ShapeIndex::SUBDIVIDED) return true;
+  if (relation == S2CellRelation::SUBDIVIDED) return true;
 
   // If "target" is an index cell, there is an intersection because index cells
   // are created only if they have at least one edge or they are entirely
@@ -584,11 +627,11 @@ bool S2Loop::Contains(const MutableS2ShapeIndex::Iterator& it,
 }
 
 void S2Loop::Encode(Encoder* const encoder) const {
-  encoder->Ensure(num_vertices_ * sizeof(*vertices_) + 20);  // sufficient
+  encoder->Ensure(num_vertices_ * sizeof(vertices_[0]) + 20);  // sufficient
 
   encoder->put8(kCurrentLosslessEncodingVersionNumber);
   encoder->put32(num_vertices_);
-  encoder->putn(vertices_, sizeof(*vertices_) * num_vertices_);
+  encoder->putn(vertices_.get(), sizeof(vertices_[0]) * num_vertices_);
   encoder->put8(origin_inside_);
   encoder->put32(depth_);
   S2_DCHECK_GE(encoder->avail(), 0);
@@ -601,60 +644,30 @@ bool S2Loop::Decode(Decoder* const decoder) {
   unsigned char version = decoder->get8();
   switch (version) {
     case kCurrentLosslessEncodingVersionNumber:
-      return DecodeInternal(decoder, false);
+      return DecodeInternal(decoder);
   }
   return false;
 }
 
-bool S2Loop::DecodeWithinScope(Decoder* const decoder) {
-  if (decoder->avail() < sizeof(unsigned char)) return false;
-  unsigned char version = decoder->get8();
-  switch (version) {
-    case kCurrentLosslessEncodingVersionNumber:
-      return DecodeInternal(decoder, true);
-  }
-  return false;
-}
-
-bool S2Loop::DecodeInternal(Decoder* const decoder,
-                            bool within_scope) {
+bool S2Loop::DecodeInternal(Decoder* const decoder) {
   // Perform all checks before modifying vertex state. Empty loops are
   // explicitly allowed here: a newly created loop has zero vertices
   // and such loops encode and decode properly.
   if (decoder->avail() < sizeof(uint32)) return false;
   const uint32 num_vertices = decoder->get32();
-  if (num_vertices > FLAGS_s2polygon_decode_max_num_vertices) {
+  if (num_vertices > static_cast<uint32>(absl::GetFlag(
+                         FLAGS_s2polygon_decode_max_num_vertices))) {
     return false;
   }
-  if (decoder->avail() < (num_vertices * sizeof(*vertices_) +
+  if (decoder->avail() < (num_vertices * sizeof(vertices_[0]) +
                           sizeof(uint8) + sizeof(uint32))) {
     return false;
   }
   ClearIndex();
-  if (owns_vertices_) delete[] vertices_;
   num_vertices_ = num_vertices;
 
-  // x86 can do unaligned floating-point reads; however, many other
-  // platforms cannot. Do not use the zero-copy version if we are on
-  // an architecture that does not support unaligned reads, and the
-  // pointer is not correctly aligned.
-#if defined(__x86_64__) || defined(_M_X64) || defined(__i386) || \
-    defined(_M_IX86)
-  bool is_misaligned = false;
-#else
-  bool is_misaligned =
-      reinterpret_cast<intptr_t>(decoder->ptr()) % sizeof(double) != 0;
-#endif
-  if (within_scope && !is_misaligned) {
-    vertices_ = const_cast<S2Point *>(reinterpret_cast<const S2Point*>(
-                    decoder->ptr()));
-    decoder->skip(num_vertices_ * sizeof(*vertices_));
-    owns_vertices_ = false;
-  } else {
-    vertices_ = new S2Point[num_vertices_];
-    decoder->getn(vertices_, num_vertices_ * sizeof(*vertices_));
-    owns_vertices_ = true;
-  }
+  vertices_ = make_unique<S2Point[]>(num_vertices_);
+  decoder->getn(vertices_.get(), num_vertices_ * sizeof(vertices_[0]));
   origin_inside_ = decoder->get8();
   depth_ = decoder->get32();
   if (!bound_.Decode(decoder)) return false;
@@ -675,8 +688,8 @@ bool S2Loop::DecodeInternal(Decoder* const decoder,
 // loops (Contains, Intersects, or CompareBoundary).
 class LoopRelation {
  public:
-  LoopRelation() {}
-  virtual ~LoopRelation() {}
+  LoopRelation() = default;
+  virtual ~LoopRelation() = default;
 
   // Optionally, a_target() and b_target() can specify an early-exit condition
   // for the loop relation.  If any point P is found such that
@@ -700,7 +713,7 @@ class LoopRelation {
   virtual int b_crossing_target() const = 0;
 
   // Given a vertex "ab1" that is shared between the two loops, return true if
-  // the two associated wedges (a0, ab1, b2) and (b0, ab1, b2) are equivalent
+  // the two associated wedges (a0, ab1, a2) and (b0, ab1, b2) are equivalent
   // to an edge crossing.  The loop relation is also allowed to maintain its
   // own internal state, and can return true if it observes any sequence of
   // wedges that are equivalent to an edge crossing.
@@ -1029,7 +1042,7 @@ class ContainsRelation : public LoopRelation {
   bool found_shared_vertex_;
 };
 
-bool S2Loop::Contains(const S2Loop* b) const {
+bool S2Loop::Contains(const S2Loop& b) const {
   // For this loop A to contains the given loop B, all of the following must
   // be true:
   //
@@ -1045,17 +1058,17 @@ bool S2Loop::Contains(const S2Loop* b) const {
   // The second part of (3) is necessary to detect the case of two loops whose
   // union is the entire sphere, i.e. two loops that contains each other's
   // boundaries but not each other's interiors.
-  if (!subregion_bound_.Contains(b->bound_)) return false;
+  if (!subregion_bound_.Contains(b.bound_)) return false;
 
   // Special cases to handle either loop being empty or full.
-  if (is_empty_or_full() || b->is_empty_or_full()) {
-    return is_full() || b->is_empty();
+  if (is_empty_or_full() || b.is_empty_or_full()) {
+    return is_full() || b.is_empty();
   }
 
   // Check whether there are any edge crossings, and also check the loop
   // relationship at any shared vertices.
   ContainsRelation relation;
-  if (HasCrossingRelation(*this, *b, &relation)) return false;
+  if (HasCrossingRelation(*this, b, &relation)) return false;
 
   // There are no crossings, and if there are any shared vertices then A
   // contains B locally at each shared vertex.
@@ -1064,18 +1077,18 @@ bool S2Loop::Contains(const S2Loop* b) const {
   // Since there are no edge intersections or shared vertices, we just need to
   // test condition (3) above.  We can skip this test if we discovered that A
   // contains at least one point of B while checking for edge crossings.
-  if (!Contains(b->vertex(0))) return false;
+  if (!Contains(b.vertex(0))) return false;
 
   // We still need to check whether (A union B) is the entire sphere.
   // Normally this check is very cheap due to the bounding box precondition.
-  if ((b->subregion_bound_.Contains(bound_) ||
-       b->bound_.Union(bound_).is_full()) && b->Contains(vertex(0))) {
+  if ((b.subregion_bound_.Contains(bound_) ||
+       b.bound_.Union(bound_).is_full()) &&
+      b.Contains(vertex(0))) {
     return false;
   }
   return true;
 }
 
-
 // Loop relation for Intersects().
 class IntersectsRelation : public LoopRelation {
  public:
@@ -1097,16 +1110,16 @@ class IntersectsRelation : public LoopRelation {
   bool found_shared_vertex_;
 };
 
-bool S2Loop::Intersects(const S2Loop* b) const {
+bool S2Loop::Intersects(const S2Loop& b) const {
   // a->Intersects(b) if and only if !a->Complement()->Contains(b).
   // This code is similar to Contains(), but is optimized for the case
   // where both loops enclose less than half of the sphere.
-  if (!bound_.Intersects(b->bound_)) return false;
+  if (!bound_.Intersects(b.bound_)) return false;
 
   // Check whether there are any edge crossings, and also check the loop
   // relationship at any shared vertices.
   IntersectsRelation relation;
-  if (HasCrossingRelation(*this, *b, &relation)) return true;
+  if (HasCrossingRelation(*this, b, &relation)) return true;
   if (relation.found_shared_vertex()) return false;
 
   // Since there are no edge intersections or shared vertices, the loops
@@ -1117,13 +1130,12 @@ bool S2Loop::Intersects(const S2Loop* b) const {
 
   // Check whether A contains B, or A and B contain each other's boundaries.
   // (Note that A contains all the vertices of B in either case.)
-  if (subregion_bound_.Contains(b->bound_) ||
-      bound_.Union(b->bound_).is_full()) {
-    if (Contains(b->vertex(0))) return true;
+  if (subregion_bound_.Contains(b.bound_) || bound_.Union(b.bound_).is_full()) {
+    if (Contains(b.vertex(0))) return true;
   }
   // Check whether B contains A.
-  if (b->subregion_bound_.Contains(bound_)) {
-    if (b->Contains(vertex(0))) return true;
+  if (b.subregion_bound_.Contains(bound_)) {
+    if (b.Contains(vertex(0))) return true;
   }
   return false;
 }
@@ -1187,96 +1199,96 @@ class CompareBoundaryRelation : public LoopRelation {
   bool excludes_edge_;        // True if any edge of B is excluded by A.
 };
 
-int S2Loop::CompareBoundary(const S2Loop* b) const {
-  S2_DCHECK(!is_empty() && !b->is_empty());
-  S2_DCHECK(!b->is_full() || !b->is_hole());
+int S2Loop::CompareBoundary(const S2Loop& b) const {
+  S2_DCHECK(!is_empty() && !b.is_empty());
+  S2_DCHECK(!b.is_full() || !b.is_hole());
 
   // The bounds must intersect for containment or crossing.
-  if (!bound_.Intersects(b->bound_)) return -1;
+  if (!bound_.Intersects(b.bound_)) return -1;
 
   // Full loops are handled as though the loop surrounded the entire sphere.
   if (is_full()) return 1;
-  if (b->is_full()) return -1;
+  if (b.is_full()) return -1;
 
   // Check whether there are any edge crossings, and also check the loop
   // relationship at any shared vertices.
-  CompareBoundaryRelation relation(b->is_hole());
-  if (HasCrossingRelation(*this, *b, &relation)) return 0;
+  CompareBoundaryRelation relation(b.is_hole());
+  if (HasCrossingRelation(*this, b, &relation)) return 0;
   if (relation.found_shared_vertex()) {
     return relation.contains_edge() ? 1 : -1;
   }
 
   // There are no edge intersections or shared vertices, so we can check
   // whether A contains an arbitrary vertex of B.
-  return Contains(b->vertex(0)) ? 1 : -1;
+  return Contains(b.vertex(0)) ? 1 : -1;
 }
 
-bool S2Loop::ContainsNonCrossingBoundary(const S2Loop* b, bool reverse_b)
-    const {
-  S2_DCHECK(!is_empty() && !b->is_empty());
-  S2_DCHECK(!b->is_full() || !reverse_b);
+bool S2Loop::ContainsNonCrossingBoundary(const S2Loop& b,
+                                         bool reverse_b) const {
+  S2_DCHECK(!is_empty() && !b.is_empty());
+  S2_DCHECK(!b.is_full() || !reverse_b);
 
   // The bounds must intersect for containment.
-  if (!bound_.Intersects(b->bound_)) return false;
+  if (!bound_.Intersects(b.bound_)) return false;
 
   // Full loops are handled as though the loop surrounded the entire sphere.
   if (is_full()) return true;
-  if (b->is_full()) return false;
+  if (b.is_full()) return false;
 
-  int m = FindVertex(b->vertex(0));
+  int m = FindVertex(b.vertex(0));
   if (m < 0) {
     // Since vertex b0 is not shared, we can check whether A contains it.
-    return Contains(b->vertex(0));
+    return Contains(b.vertex(0));
   }
   // Otherwise check whether the edge (b0, b1) is contained by A.
-  return WedgeContainsSemiwedge(vertex(m-1), vertex(m), vertex(m+1),
-                                b->vertex(1), reverse_b);
+  return WedgeContainsSemiwedge(vertex(m - 1), vertex(m), vertex(m + 1),
+                                b.vertex(1), reverse_b);
 }
 
-bool S2Loop::ContainsNested(const S2Loop* b) const {
-  if (!subregion_bound_.Contains(b->bound_)) return false;
+bool S2Loop::ContainsNested(const S2Loop& b) const {
+  if (!subregion_bound_.Contains(b.bound_)) return false;
 
   // Special cases to handle either loop being empty or full.  Also bail out
   // when B has no vertices to avoid heap overflow on the vertex(1) call
   // below.  (This method is called during polygon initialization before the
   // client has an opportunity to call IsValid().)
-  if (is_empty_or_full() || b->num_vertices() < 2) {
-    return is_full() || b->is_empty();
+  if (is_empty_or_full() || b.num_vertices() < 2) {
+    return is_full() || b.is_empty();
   }
 
   // We are given that A and B do not share any edges, and that either one
   // loop contains the other or they do not intersect.
-  int m = FindVertex(b->vertex(1));
+  int m = FindVertex(b.vertex(1));
   if (m < 0) {
-    // Since b->vertex(1) is not shared, we can check whether A contains it.
-    return Contains(b->vertex(1));
+    // Since b.vertex(1) is not shared, we can check whether A contains it.
+    return Contains(b.vertex(1));
   }
-  // Check whether the edge order around b->vertex(1) is compatible with
+  // Check whether the edge order around b.vertex(1) is compatible with
   // A containing B.
-  return S2::WedgeContains(vertex(m-1), vertex(m), vertex(m+1),
-                                   b->vertex(0), b->vertex(2));
+  return S2::WedgeContains(vertex(m - 1), vertex(m), vertex(m + 1), b.vertex(0),
+                           b.vertex(2));
 }
 
-bool S2Loop::Equals(const S2Loop* b) const {
-  if (num_vertices() != b->num_vertices()) return false;
+bool S2Loop::Equals(const S2Loop& b) const {
+  if (num_vertices() != b.num_vertices()) return false;
   for (int i = 0; i < num_vertices(); ++i) {
-    if (vertex(i) != b->vertex(i)) return false;
+    if (vertex(i) != b.vertex(i)) return false;
   }
   return true;
 }
 
-bool S2Loop::BoundaryEquals(const S2Loop* b) const {
-  if (num_vertices() != b->num_vertices()) return false;
+bool S2Loop::BoundaryEquals(const S2Loop& b) const {
+  if (num_vertices() != b.num_vertices()) return false;
 
   // Special case to handle empty or full loops.  Since they have the same
   // number of vertices, if one loop is empty/full then so is the other.
-  if (is_empty_or_full()) return is_empty() == b->is_empty();
+  if (is_empty_or_full()) return is_empty() == b.is_empty();
 
   for (int offset = 0; offset < num_vertices(); ++offset) {
-    if (vertex(offset) == b->vertex(0)) {
+    if (vertex(offset) == b.vertex(0)) {
       // There is at most one starting offset since loop vertices are unique.
       for (int i = 0; i < num_vertices(); ++i) {
-        if (vertex(i + offset) != b->vertex(i)) return false;
+        if (vertex(i + offset) != b.vertex(i)) return false;
       }
       return true;
     }
@@ -1322,7 +1334,7 @@ static bool MatchBoundaries(const S2Loop& a, const S2Loop& b, int a_offset,
   // explored to avoid duplicating work.
 
   vector<pair<int, int>> pending;
-  set<pair<int, int>> done;
+  flat_hash_set<pair<int, int>> done;
   pending.push_back(std::make_pair(0, 0));
   while (!pending.empty()) {
     int i = pending.back().first;
@@ -1404,17 +1416,16 @@ bool S2Loop::DecodeCompressed(Decoder* decoder, int snap_level) {
     return false;
   }
   if (unsigned_num_vertices == 0 ||
-      unsigned_num_vertices > FLAGS_s2polygon_decode_max_num_vertices) {
+      unsigned_num_vertices > static_cast<uint32>(absl::GetFlag(
+                                  FLAGS_s2polygon_decode_max_num_vertices))) {
     return false;
   }
   ClearIndex();
-  if (owns_vertices_) delete[] vertices_;
   num_vertices_ = unsigned_num_vertices;
-  vertices_ = new S2Point[num_vertices_];
-  owns_vertices_ = true;
+  vertices_ = make_unique<S2Point[]>(num_vertices_);
 
   if (!S2DecodePointsCompressed(decoder, snap_level,
-                                MakeSpan(vertices_, num_vertices_))) {
+                                MakeSpan(vertices_.get(), num_vertices_))) {
     return false;
   }
   uint32 properties_uint32;
@@ -1463,18 +1474,14 @@ std::bitset<kNumProperties> S2Loop::GetCompressedEncodingProperties() const {
 }
 
 /* static */
-std::unique_ptr<S2Loop> S2Loop::MakeRegularLoop(const S2Point& center,
-                                                S1Angle radius,
-                                                int num_vertices) {
-  Matrix3x3_d m;
-  S2::GetFrame(center, &m);  // TODO(ericv): Return by value
-  return MakeRegularLoop(m, radius, num_vertices);
+unique_ptr<S2Loop> S2Loop::MakeRegularLoop(const S2Point& center,
+                                           S1Angle radius, int num_vertices) {
+  return MakeRegularLoop(S2::GetFrame(center), radius, num_vertices);
 }
 
 /* static */
-std::unique_ptr<S2Loop> S2Loop::MakeRegularLoop(const Matrix3x3_d& frame,
-                                                S1Angle radius,
-                                                int num_vertices) {
+unique_ptr<S2Loop> S2Loop::MakeRegularLoop(const Matrix3x3_d& frame,
+                                           S1Angle radius, int num_vertices) {
   // We construct the loop in the given frame coordinates, with the center at
   // (0, 0, 1).  For a loop of radius "r", the loop vertices have the form
   // (x, y, z) where x^2 + y^2 = sin(r) and z = cos(r).  The distance on the
