@@ -18,17 +18,21 @@
 #ifndef S2_S2REGION_COVERER_H_
 #define S2_S2REGION_COVERER_H_
 
+#include <algorithm>
 #include <cstddef>
 #include <new>
 #include <queue>
 #include <utility>
 #include <vector>
 
+#include "absl/base/casts.h"
 #include "absl/base/macros.h"
 #include "s2/_fp_contract_off.h"
 #include "s2/s2cell.h"
 #include "s2/s2cell_id.h"
 #include "s2/s2cell_union.h"
+#include "s2/s2point.h"
+#include "s2/s2region.h"
 
 class S2Region;
 
@@ -179,7 +183,7 @@ class S2RegionCoverer {
   // Returns an S2CellUnion that covers (GetCovering) or is contained within
   // (GetInteriorCovering) the given region and satisfies the current options.
   //
-  // Note that if options().min_level() > 0 or options().level_mod() > 1, the
+  // Note that if options().min_level() > 0 or options().level_mod() > 1, then
   // by definition the S2CellUnion may not be normalized, i.e. there may be
   // groups of four child cells that can be replaced by their parent cell.
   S2CellUnion GetCovering(const S2Region& region);
@@ -248,15 +252,15 @@ class S2RegionCoverer {
  private:
   struct Candidate {
     void* operator new(std::size_t size, std::size_t max_children) {
-      // dd: CRAN reports a sanitizer error when using this magic, so
-      // we just hard code for the maximum size the comments suggest it will be
-      // return ::operator new (size + max_children * sizeof(Candidate *));
-      return ::operator new (size);
+      // CRAN edit: the sanitizer doesn't understand [0], so we just fix the
+      // size at the
+      // widest possible size.
+      // https://github.com/r-spatial/s2/pull/238
+      // https://github.com/r-spatial/s2/pull/275
+      return ::operator new(size);
     }
 
-    void operator delete(void* p) {
-      ::operator delete (p);
-    }
+    void operator delete(void* p) { ::operator delete(p); }
 
     Candidate(const S2Cell& cell, const std::size_t max_children)
         : cell(cell), is_terminal(max_children == 0) {
@@ -269,10 +273,11 @@ class S2RegionCoverer {
     ~Candidate() = default;
 
     S2Cell cell;
-    bool is_terminal;        // Cell should not be expanded further.
-    int num_children = 0;    // Number of children that intersect the region.
-    // dd: To avoid a sanitizer, we hard-code 64 here rather than rely on
-    // flexible array member magic.
+    bool is_terminal;      // Cell should not be expanded further.
+    int num_children = 0;  // Number of children that intersect the region.
+    // CRAN edit: the sanitizer doesn't understand [0], so we just fix the size
+    // at the widest possible size. https://github.com/r-spatial/s2/pull/238
+    // https://github.com/r-spatial/s2/pull/275
     Candidate* children[64];  // Actual size may be 0, 4, 16, or 64 elements.
   };
 
@@ -348,7 +353,8 @@ class S2RegionCoverer {
     }
   };
   typedef std::priority_queue<QueueEntry, std::vector<QueueEntry>,
-                              CompareQueueEntries> CandidateQueue;
+                              CompareQueueEntries>
+      CandidateQueue;
   CandidateQueue pq_;
 
   // True if we're computing an interior covering.
