@@ -15,28 +15,41 @@
 
 #include "s2/s2text_format.h"
 
+#include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "s2/base/logging.h"
-#include "s2/base/stringprintf.h"
-#include "s2/strings/serialize.h"
-#include "absl/memory/memory.h"
+#include "absl/strings/ascii.h"
+#include "absl/strings/numbers.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/string_view.h"
-#include "absl/strings/strip.h"
+#include "absl/types/span.h"
+
 #include "s2/mutable_s2shape_index.h"
+#include "s2/s1angle.h"
+#include "s2/s2cell_id.h"
+#include "s2/s2cell_union.h"
+#include "s2/s2debug.h"
 #include "s2/s2latlng.h"
+#include "s2/s2latlng_rect.h"
 #include "s2/s2lax_polygon_shape.h"
 #include "s2/s2lax_polyline_shape.h"
 #include "s2/s2loop.h"
+#include "s2/s2point.h"
 #include "s2/s2point_vector_shape.h"
 #include "s2/s2polygon.h"
 #include "s2/s2polyline.h"
+#include "s2/s2shape.h"
+#include "s2/s2shape_index.h"
 
-using absl::make_unique;
+using absl::Span;
 using absl::string_view;
+using std::make_unique;
 using std::pair;
+using std::string;
 using std::unique_ptr;
 using std::vector;
 
@@ -51,12 +64,6 @@ static vector<string_view> SplitString(string_view str, char separator) {
   return result;
 }
 
-static bool ParseDouble(const std::string& str, double* value) {
-  char* end_ptr = nullptr;
-  *value = strtod(str.c_str(), &end_ptr);
-  return end_ptr && *end_ptr == 0;
-}
-
 vector<S2LatLng> ParseLatLngsOrDie(string_view str) {
   vector<S2LatLng> latlngs;
   S2_CHECK(ParseLatLngs(str, &latlngs)) << ": str == \"" << str << "\"";
@@ -64,13 +71,13 @@ vector<S2LatLng> ParseLatLngsOrDie(string_view str) {
 }
 
 bool ParseLatLngs(string_view str, vector<S2LatLng>* latlngs) {
-  vector<pair<std::string, std::string>> ps;
-  if (!strings::DictionaryParse(str, &ps)) return false;
-  for (const auto& p : ps) {
-    double lat;
-    if (!ParseDouble(p.first, &lat)) return false;
-    double lng;
-    if (!ParseDouble(p.second, &lng)) return false;
+  for (const string_view lat_lng_str :
+       absl::StrSplit(str, ',', absl::SkipEmpty())) {
+    const vector<string_view> lat_lng = absl::StrSplit(lat_lng_str, ':');
+    if (lat_lng.size() != 2) return false;
+    double lat, lng;
+    if (!absl::SimpleAtod(lat_lng[0], &lat)) return false;
+    if (!absl::SimpleAtod(lat_lng[1], &lng)) return false;
     latlngs->push_back(S2LatLng::FromDegrees(lat, lng));
   }
   return true;
@@ -105,7 +112,7 @@ bool MakePoint(string_view str, S2Point* point) {
 }
 
 bool MakeLatLng(string_view str, S2LatLng* latlng) {
-  std::vector<S2LatLng> latlngs;
+  vector<S2LatLng> latlngs;
   if (!ParseLatLngs(str, &latlngs) || latlngs.size() != 1) return false;
   *latlng = latlngs[0];
   return true;
@@ -127,7 +134,7 @@ bool MakeLatLngRect(string_view str, S2LatLngRect* rect) {
   vector<S2LatLng> latlngs;
   if (!ParseLatLngs(str, &latlngs) || latlngs.empty()) return false;
   *rect = S2LatLngRect::FromPoint(latlngs[0]);
-  for (int i = 1; i < latlngs.size(); ++i) {
+  for (size_t i = 1; i < latlngs.size(); ++i) {
     rect->AddPoint(latlngs[i]);
   }
   return rect;
@@ -183,10 +190,6 @@ bool MakeLoop(string_view str, unique_ptr<S2Loop>* loop,
   return true;
 }
 
-std::unique_ptr<S2Loop> MakeLoop(string_view str, S2Debug debug_override) {
-  return MakeLoopOrDie(str, debug_override);
-}
-
 unique_ptr<S2Polyline> MakePolylineOrDie(string_view str,
                                          S2Debug debug_override) {
   unique_ptr<S2Polyline> polyline;
@@ -203,11 +206,6 @@ bool MakePolyline(string_view str, unique_ptr<S2Polyline>* polyline,
   return true;
 }
 
-std::unique_ptr<S2Polyline> MakePolyline(string_view str,
-                                         S2Debug debug_override) {
-  return MakePolylineOrDie(str, debug_override);
-}
-
 unique_ptr<S2LaxPolylineShape> MakeLaxPolylineOrDie(string_view str) {
   unique_ptr<S2LaxPolylineShape> lax_polyline;
   S2_CHECK(MakeLaxPolyline(str, &lax_polyline)) << ": str == \"" << str << "\"";
@@ -222,10 +220,6 @@ bool MakeLaxPolyline(string_view str,
   return true;
 }
 
-std::unique_ptr<S2LaxPolylineShape> MakeLaxPolyline(string_view str) {
-  return MakeLaxPolylineOrDie(str);
-}
-
 static bool InternalMakePolygon(string_view str,
                                 S2Debug debug_override,
                                 bool normalize_loops,
@@ -234,7 +228,7 @@ static bool InternalMakePolygon(string_view str,
   vector<string_view> loop_strs = SplitString(str, ';');
   vector<unique_ptr<S2Loop>> loops;
   for (const auto& loop_str : loop_strs) {
-    std::unique_ptr<S2Loop> loop;
+    unique_ptr<S2Loop> loop;
     if (!MakeLoop(loop_str, &loop, debug_override)) return false;
     // Don't normalize loops that were explicitly specified as "full".
     if (normalize_loops && !loop->is_full()) loop->Normalize();
@@ -257,11 +251,6 @@ bool MakePolygon(string_view str, unique_ptr<S2Polygon>* polygon,
   return InternalMakePolygon(str, debug_override, true, polygon);
 }
 
-std::unique_ptr<S2Polygon> MakePolygon(string_view str,
-                                       S2Debug debug_override) {
-  return MakePolygonOrDie(str, debug_override);
-}
-
 unique_ptr<S2Polygon> MakeVerbatimPolygonOrDie(string_view str) {
   unique_ptr<S2Polygon> polygon;
   S2_CHECK(MakeVerbatimPolygon(str, &polygon)) << ": str == \"" << str << "\"";
@@ -272,10 +261,6 @@ bool MakeVerbatimPolygon(string_view str, unique_ptr<S2Polygon>* polygon) {
   return InternalMakePolygon(str, S2Debug::ALLOW, false, polygon);
 }
 
-std::unique_ptr<S2Polygon> MakeVerbatimPolygon(string_view str) {
-  return MakeVerbatimPolygonOrDie(str);
-}
-
 unique_ptr<S2LaxPolygonShape> MakeLaxPolygonOrDie(string_view str) {
   unique_ptr<S2LaxPolygonShape> lax_polygon;
   S2_CHECK(MakeLaxPolygon(str, &lax_polygon)) << ": str == \"" << str << "\"";
@@ -299,17 +284,13 @@ bool MakeLaxPolygon(string_view str,
   return true;
 }
 
-std::unique_ptr<S2LaxPolygonShape> MakeLaxPolygon(string_view str) {
-  return MakeLaxPolygonOrDie(str);
-}
-
 unique_ptr<MutableS2ShapeIndex> MakeIndexOrDie(string_view str) {
   auto index = make_unique<MutableS2ShapeIndex>();
   S2_CHECK(MakeIndex(str, &index)) << ": str == \"" << str << "\"";
   return index;
 }
 
-bool MakeIndex(string_view str, std::unique_ptr<MutableS2ShapeIndex>* index) {
+bool MakeIndex(string_view str, unique_ptr<MutableS2ShapeIndex>* index) {
   vector<string_view> strs = absl::StrSplit(str, '#');
   S2_DCHECK_EQ(3, strs.size()) << "Must contain two # characters: " << str;
 
@@ -323,64 +304,68 @@ bool MakeIndex(string_view str, std::unique_ptr<MutableS2ShapeIndex>* index) {
     (*index)->Add(make_unique<S2PointVectorShape>(std::move(points)));
   }
   for (const auto& line_str : SplitString(strs[1], '|')) {
-    std::unique_ptr<S2LaxPolylineShape> lax_polyline;
+    unique_ptr<S2LaxPolylineShape> lax_polyline;
     if (!MakeLaxPolyline(line_str, &lax_polyline)) return false;
-    (*index)->Add(unique_ptr<S2Shape>(lax_polyline.release()));
+    (*index)->Add(std::move(lax_polyline));
   }
   for (const auto& polygon_str : SplitString(strs[2], '|')) {
-    std::unique_ptr<S2LaxPolygonShape> lax_polygon;
+    unique_ptr<S2LaxPolygonShape> lax_polygon;
     if (!MakeLaxPolygon(polygon_str, &lax_polygon)) return false;
-    (*index)->Add(unique_ptr<S2Shape>(lax_polygon.release()));
+    (*index)->Add(std::move(lax_polygon));
   }
   return true;
 }
 
-std::unique_ptr<MutableS2ShapeIndex> MakeIndex(string_view str) {
-  return MakeIndexOrDie(str);
+static void AppendVertex(const S2LatLng& ll, string* out,
+                         bool roundtrip_precision = false) {
+  if (roundtrip_precision) {
+    absl::StrAppendFormat(out, "%.17g:%.17g", ll.lat().degrees(),
+                          ll.lng().degrees());
+  } else {
+    absl::StrAppendFormat(out, "%.15g:%.15g", ll.lat().degrees(),
+                          ll.lng().degrees());
+  }
 }
 
-static void AppendVertex(const S2LatLng& ll, std::string* out) {
-  StringAppendF(out, "%.15g:%.15g", ll.lat().degrees(), ll.lng().degrees());
-}
-
-static void AppendVertex(const S2Point& p, std::string* out) {
+static void AppendVertex(const S2Point& p, string* out,
+                         bool roundtrip_precision = false) {
   S2LatLng ll(p);
-  return AppendVertex(ll, out);
+  return AppendVertex(ll, out, roundtrip_precision);
 }
 
-static void AppendVertices(const S2Point* v, int n, std::string* out) {
+static void AppendVertices(const S2Point* v, int n, string* out) {
   for (int i = 0; i < n; ++i) {
     if (i > 0) *out += ", ";
     AppendVertex(v[i], out);
   }
 }
 
-std::string ToString(const S2Point& point) {
-  std::string out;
+string ToString(const S2Point& point) {
+  string out;
   AppendVertex(point, &out);
   return out;
 }
 
-std::string ToString(const S2LatLng& latlng) {
-  std::string out;
+string ToString(const S2LatLng& latlng) {
+  string out;
   AppendVertex(latlng, &out);
   return out;
 }
 
-std::string ToString(const S2LatLngRect& rect) {
-  std::string out;
+string ToString(const S2LatLngRect& rect) {
+  string out;
   AppendVertex(rect.lo(), &out);
   out += ", ";
   AppendVertex(rect.hi(), &out);
   return out;
 }
 
-std::string ToString(const S2CellId& cell_id) {
+string ToString(const S2CellId cell_id) {
   return cell_id.ToString();
 }
 
-std::string ToString(const S2CellUnion& cell_union) {
-  std::string out;
+string ToString(const S2CellUnion& cell_union) {
+  string out;
   for (S2CellId cell_id : cell_union) {
     if (!out.empty()) out += ", ";
     out += cell_id.ToString();
@@ -388,80 +373,94 @@ std::string ToString(const S2CellUnion& cell_union) {
   return out;
 }
 
-std::string ToString(const S2Loop& loop) {
+string ToString(const S2Loop& loop) {
   if (loop.is_empty()) {
     return "empty";
   } else if (loop.is_full()) {
     return "full";
   }
-  std::string out;
+  string out;
   if (loop.num_vertices() > 0) {
     AppendVertices(&loop.vertex(0), loop.num_vertices(), &out);
   }
   return out;
 }
 
-std::string ToString(S2PointLoopSpan loop) {
-  // S2Shape represents the full loop as a loop with no vertices.
-  // There is no representation of the empty loop.
-  if (loop.empty()) {
-    return "full";
-  }
-  std::string out;
-  AppendVertices(loop.data(), loop.size(), &out);
-  return out;
-}
-
-std::string ToString(const S2Polyline& polyline) {
-  std::string out;
+string ToString(const S2Polyline& polyline) {
+  string out;
   if (polyline.num_vertices() > 0) {
     AppendVertices(&polyline.vertex(0), polyline.num_vertices(), &out);
   }
   return out;
 }
 
-std::string ToString(const S2Polygon& polygon, const char* loop_separator) {
+string ToString(const S2Polygon& polygon, string_view loop_separator) {
   if (polygon.is_empty()) {
     return "empty";
   } else if (polygon.is_full()) {
     return "full";
   }
-  std::string out;
+  string out;
   for (int i = 0; i < polygon.num_loops(); ++i) {
-    if (i > 0) out += loop_separator;
+    if (i > 0) absl::StrAppend(&out, loop_separator);
     const S2Loop& loop = *polygon.loop(i);
     AppendVertices(&loop.vertex(0), loop.num_vertices(), &out);
   }
   return out;
 }
 
-std::string ToString(const vector<S2Point>& points) {
-  std::string out;
+string ToString(Span<const S2Point> points) {
+  string out;
   AppendVertices(points.data(), points.size(), &out);
   return out;
 }
 
-std::string ToString(const vector<S2LatLng>& latlngs) {
-  std::string out;
-  for (int i = 0; i < latlngs.size(); ++i) {
+string ToString(Span<const S2LatLng> latlngs) {
+  string out;
+  for (size_t i = 0; i < latlngs.size(); ++i) {
     if (i > 0) out += ", ";
     AppendVertex(latlngs[i], &out);
   }
   return out;
 }
 
-std::string ToString(const S2LaxPolylineShape& polyline) {
-  std::string out;
+string ToString(const S2Shape& shape) {
+  string out;
+  if (shape.dimension() == 1) out += "# ";
+  if (shape.dimension() == 2) out += "## ";
+
+  int nchain = 0;
+  for (const auto& chain : shape.chains()) {
+    if (nchain++ > 0) {
+      out += " | ";
+    }
+
+    int nvertex = 0;
+    for (const S2Point& vertex : shape.vertices(chain)) {
+      if (nvertex++ > 0) {
+        out += ", ";
+      }
+      AppendVertex(vertex, &out);
+    }
+  }
+
+  if (shape.dimension() == 1) out += " #";
+  if (shape.dimension() == 0) out += " ##";
+  return out;
+}
+
+string ToString(const S2LaxPolylineShape& polyline) {
+  string out;
   if (polyline.num_vertices() > 0) {
     AppendVertices(&polyline.vertex(0), polyline.num_vertices(), &out);
   }
   return out;
 }
 
-std::string ToString(const S2LaxPolygonShape& polygon, const char* loop_separator) {
-  std::string out;
+string ToString(const S2LaxPolygonShape& polygon, string_view loop_separator) {
+  string out;
   for (int i = 0; i < polygon.num_loops(); ++i) {
-    if (i > 0) out += loop_separator;
+    if (i > 0) absl::StrAppend(&out, loop_separator);
     int n = polygon.num_loop_vertices(i);
     if (n == 0) {
       out += "full";
@@ -472,8 +471,8 @@ std::string ToString(const S2LaxPolygonShape& polygon, const char* loop_separato
   return out;
 }
 
-std::string ToString(const S2ShapeIndex& index) {
-  std::string out;
+string ToString(const S2ShapeIndex& index, bool roundtrip_precision) {
+  string out;
   for (int dim = 0; dim < 3; ++dim) {
     if (dim > 0) out += "#";
     int count = 0;
@@ -487,13 +486,13 @@ std::string ToString(const S2ShapeIndex& index) {
           S2_DCHECK_EQ(dim, 2);
           out += "full";
         } else {
-          AppendVertex(shape->edge(chain.start).v0, &out);
+          AppendVertex(shape->edge(chain.start).v0, &out, roundtrip_precision);
         }
         int limit = chain.start + chain.length;
         if (dim != 1) --limit;
         for (int e = chain.start; e < limit; ++e) {
           out += ", ";
-          AppendVertex(shape->edge(e).v1, &out);
+          AppendVertex(shape->edge(e).v1, &out, roundtrip_precision);
         }
       }
     }
