@@ -40,7 +40,9 @@
 #include <cstddef>
 #include <cstring>
 #include <sys/types.h>
+
 #include <algorithm>
+#include <cstdint>
 #include <iterator>
 #include <memory>
 #include <ostream>  // NOLINT
@@ -48,11 +50,12 @@
 #include <type_traits>
 #include <utility>
 
+#include "absl/base/macros.h"
+#include "absl/meta/type_traits.h"
+
 #include "s2/base/integral_types.h"
 #include "s2/base/logging.h"
-#include "absl/base/macros.h"
 #include "s2/base/port.h"
-#include "absl/meta/type_traits.h"
 #include "s2/util/bits/bits.h"
 #include "s2/util/gtl/container_logging.h"
 
@@ -70,14 +73,14 @@ class compact_array_base {
   static const int kMaxSize = (1 << kSizeNumBits) - 1;
 
 #ifdef IS_LITTLE_ENDIAN
-  uint32 size_        : kSizeNumBits;      // number of valid items in the array
-  uint32 capacity_    : kCapacityNumBits;  // allocated array size
-  uint32 is_exponent_ : 1;                 // whether capacity_ is an exponent
+  uint32 size_ : kSizeNumBits;          // number of valid items in the array
+  uint32 capacity_ : kCapacityNumBits;  // allocated array size
+  uint32 is_exponent_ : 1;              // whether capacity_ is an exponent
 
   // This object might share memory representation (ie. union) with
   // other data structures. We reserved the DO_NOT_USE (32nd bit in
   // little endian format) to be used as a tag.
-  uint32 DO_NOT_USE   : 1;
+  uint32 DO_NOT_USE : 1;
 #else
   uint32 DO_NOT_USE   : 1;
   uint32 is_exponent_ : 1;
@@ -86,10 +89,8 @@ class compact_array_base {
 #endif
 
   // Opportunistically consider allowing inlined elements.
-  // dd: this has to be disabled to pass CRAN checks, since there is a
-  // (potentially) zero-length array that is not the last element of the class (so
-  // this can't be silenced using __extension__)
-#if defined(_LP64) && defined(__GNUC__) && false
+  // Edit for CRAN: This is not supported by -Wpedantic
+#if false && defined(_LP64) && defined(__GNUC__)
   // With 64-bit pointers, our approach is to form a 16-byte struct:
   //   [5 bytes for size, capacity, is_exponent and is_inlined]
   //   [3 bytes of padding or inlined elements]
@@ -124,7 +125,7 @@ class compact_array_base {
   char unused_padding_[kUnusedPaddingBytes];
 
   // inlined_elements_ stores the first N elements, potentially as few as zero.
-  __extension__ char inlined_elements_[3 - kUnusedPaddingBytes];
+  char inlined_elements_[3 - kUnusedPaddingBytes];
 
   // compact_array_base itself is at least as aligned as a T* because of the
   // T* member inside this union. The only reason to split inlined_elements_
@@ -148,7 +149,8 @@ class compact_array_base {
     return const_cast<compact_array_base<T, A>*>(this)->Array();
   }
 
-  typedef typename A::template rebind<T>::other value_allocator_type;
+  using value_allocator_type =
+      typename std::allocator_traits<A>::template rebind_alloc<T>;
 
  public:
   typedef T                                     value_type;
@@ -157,7 +159,7 @@ class compact_array_base {
   typedef const value_type*                     const_pointer;
   typedef value_type&                           reference;
   typedef const value_type&                     const_reference;
-  typedef uint32                                size_type;
+  typedef uint32 size_type;
   typedef ptrdiff_t                             difference_type;
 
   typedef value_type*                           iterator;
@@ -276,6 +278,15 @@ class compact_array_base {
     insert(p, first, last, Int());
   }
 
+  template <typename... Args>
+  reference emplace_back(Args&&... args) {
+    return *Insert(end(), value_type(std::forward<Args>(args)...));
+  }
+  template <typename... Args>
+  iterator emplace(const_iterator p, Args&&... args) {
+    return Insert(p, value_type(std::forward<Args>(args)...));
+  }
+
   iterator erase(const_iterator p) {
     size_type index = p - begin();
     erase_aux(p, 1);
@@ -357,6 +368,13 @@ class compact_array_base {
     set_size(n);
   }
 
+  template <typename H>
+  friend H AbslHashValue(H h, const compact_array_base& v) {
+    return H::combine(
+        H::combine_contiguous(std::move(h), v.ConstArray(), v.size()),
+        v.size());
+  }
+
  private:                               // Low-level helper functions.
   void set_size(size_type n) {
     S2_DCHECK_LE(n, capacity());
@@ -396,10 +414,7 @@ class compact_array_base {
     value_allocator_type allocator;
 
     T* new_ptr = allocator.allocate(capacity());
-    // dd: this modification fixes a ASAN/UBSAN error, because
-    // when old_capacity is 0, Array() is nullptr, which is UB
-    // for memcpy.
-    if (old_capacity > 0) {
+    if (old_capacity != 0) {
       memcpy(new_ptr, Array(), old_capacity * sizeof(T));
       allocator.deallocate(Array(), old_capacity);
     }
