@@ -473,7 +473,7 @@ test_that("s2_snap_to_grid() works", {
 test_that("s2_buffer() works", {
   # create a hemisphere!
   ply <- s2_buffer_cells("POINT (0 0)", distance = pi / 2, radius = 1)
-  expect_near(s2_area(ply, radius = 1), 4 * pi / 2, epsilon = 0.1)
+  expect_near(s2_area(ply, radius = 1), 4 * pi / 2, epsilon = 0.2)
 })
 
 test_that("s2_simplify() works", {
@@ -609,14 +609,9 @@ test_that("s2_rebuild() works", {
 })
 
 test_that("real data survives the S2BooleanOperation", {
-  # the 32-bit Solaris build results in some of the roundtripped
-  # edges becoming degenerate. Rather than pass check = FALSE to
-  # as_s2_geography(), just skip this on Solaris
-  skip_on_os("solaris")
-
   for (continent in unique(s2::s2_data_tbl_countries$continent)) {
     # this is primarily a test of the S2BooleanOperation -> Geography constructor
-    unioned <- expect_s3_class(s2_coverage_union_agg(s2_data_countries(continent)), "s2_geography")
+    unioned <- expect_s3_class(s2_union_agg(s2_data_countries(continent)), "s2_geography")
 
     # this is a test of RGeography::Export() on potentially complex polygons
     exported <- expect_length(s2_as_binary(unioned), 1)
